"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Batch = void 0;
var express = require("express");
var Q = require("q");
var ajaxHelper_1 = require("../../Helpers/ajaxHelper");
var Batch;
(function (Batch) {
    "use strict";
    Batch.serverRouter = express.Router();
    Batch.serverRouter.post("/", function (req, res, next) {
        var body = req.body;
        if (body && body.requests) {
            var requests = body.requests.map(function (request) {
                return ajaxHelper_1.AjaxHelper.loopbackAjax(req, request.relativeUrl || request.url, request.httpMethod).then(function (response) {
                    if (!response) {
                        return {
                            httpStatusCode: "500",
                            content: {
                                message: "No response"
                            }
                        };
                    }
                    return ajaxHelper_1.AjaxHelper.getJsonData(response).then(function (body) {
                        return {
                            httpStatusCode: response.statusCode,
                            headers: response.headers,
                            content: body,
                        };
                    });
                });
            });
            Q.allSettled(requests).then(function (responses) {
                var someFailures = responses.some(function (response) { return Number(response.value.httpStatusCode) > 400; });
                var body = responses.map(function (response) { return response.value; });
                res.status(someFailures ? 400 : 200).json({ responses: body });
            });
        }
        else {
            next();
        }
    });
})(Batch = exports.Batch || (exports.Batch = {}));
//# sourceMappingURL=batch.js.map