"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Locks = void 0;
var constants_1 = require("../../Helpers/constants");
var urlParams_1 = require("./urlParams");
var Locks;
(function (Locks) {
    "use strict";
    function mapLocks(resourceId, locks) {
        return (locks || []).map(function (lock) {
            return mapLock(resourceId, lock.name, lock.properties);
        });
    }
    function mapLock(resourceId, name, properties) {
        return {
            id: resourceId + "/provider/Microsoft.Authorization/locks/" + name,
            name: name,
            type: "Microsoft.Authorization/locks",
            properties: properties
        };
    }
    function updateLock(locks, newLock) {
        var existingLocks = locks.filter(function (item) { return item.name.toLowerCase() === newLock.name.toLowerCase(); });
        if (existingLocks) {
            for (var i = 0; i < existingLocks.length; i++) {
                locks.splice(locks.indexOf(existingLocks[i]), 1);
            }
        }
        locks.push(newLock);
        return locks;
    }
    /**
     * Initializes the routes.
     *
     * @param app The express app.
     */
    function init(app) {
        var subscriptionLocksUri = "/subscriptions/:" + urlParams_1.UrlParams.Params.subscriptionId + "/providers/Microsoft.Authorization/locks";
        app.get(subscriptionLocksUri, function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            if (mockSubscriptionId) {
                var armManager = req.app.armManager;
                var subscriptionData = armManager.getSubscriptionData(mockSubscriptionId);
                if (subscriptionData) {
                    var subscription = armManager.getSubscription(mockSubscriptionId);
                    res.status(constants_1.Constants.HttpStatusCode.Ok).json(mapLocks(subscription.id, subscriptionData.locks));
                }
                else {
                    res.sendStatus(constants_1.Constants.HttpStatusCode.NotFound);
                }
            }
            else {
                next();
            }
        });
        var subscriptionLockWithIdUri = subscriptionLocksUri + "/:id";
        app.put(subscriptionLockWithIdUri, function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            if (mockSubscriptionId) {
                var id = req.params["id"].toLowerCase();
                var armManager = req.app.armManager;
                var subscriptionData = armManager.getSubscriptionData(mockSubscriptionId);
                if (subscriptionData) {
                    var subscription = armManager.getSubscription(mockSubscriptionId);
                    var newLock = mapLock(subscription.id, id, (req.body || {}).properties);
                    var locks = subscriptionData.locks || [];
                    subscriptionData.locks = updateLock(locks, newLock);
                    res.status(constants_1.Constants.HttpStatusCode.Ok).json(newLock);
                }
            }
            else {
                next();
            }
        });
        var resourceGroupLocksUri = "/subscriptions/:" + urlParams_1.UrlParams.Params.subscriptionId + "/resourcegroups/:" + urlParams_1.UrlParams.Params.resourceGroupId + "/providers/Microsoft.Authorization/locks";
        app.get(resourceGroupLocksUri, function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            var mockResourceGroupId = req[urlParams_1.UrlParams.Params.resourceGroupId];
            if (mockSubscriptionId) {
                var armManager = req.app.armManager;
                var resourceGroupData = armManager.getResourceGroupData(mockSubscriptionId, mockResourceGroupId);
                if (resourceGroupData) {
                    var resourceGroup = armManager.getResourceGroup(mockSubscriptionId, mockResourceGroupId);
                    res.status(constants_1.Constants.HttpStatusCode.Ok).json(mapLocks(resourceGroup.id, resourceGroupData.locks));
                }
                else {
                    res.sendStatus(constants_1.Constants.HttpStatusCode.NotFound);
                }
            }
            else {
                next();
            }
        });
        var resourceGroupLockWithIdUri = resourceGroupLocksUri + "/:id";
        app.put(resourceGroupLockWithIdUri, function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            var mockResourceGroupId = req[urlParams_1.UrlParams.Params.resourceGroupId];
            if (mockSubscriptionId) {
                var id = req.params["id"].toLowerCase();
                var armManager = req.app.armManager;
                var resourceGroupData = armManager.getResourceGroupData(mockSubscriptionId, mockResourceGroupId);
                if (resourceGroupData) {
                    var resourceGroup = armManager.getResourceGroup(mockSubscriptionId, mockResourceGroupId);
                    var newLock = mapLock(resourceGroup.id, id, (req.body || {}).properties);
                    var locks = resourceGroupData.locks || [];
                    resourceGroupData.locks = updateLock(locks, newLock);
                    res.status(constants_1.Constants.HttpStatusCode.Ok).json(newLock);
                }
            }
            else {
                next();
            }
        });
        var resourceLocksUri = "/subscriptions/:" + urlParams_1.UrlParams.Params.subscriptionId + "/resourcegroups/:" + urlParams_1.UrlParams.Params.resourceGroupId + "/providers/:providerId/:resourceType/:resourceId/providers/Microsoft.Authorization/locks";
        app.get(resourceLocksUri, function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            var mockResourceGroupId = req[urlParams_1.UrlParams.Params.resourceGroupId];
            if (mockSubscriptionId) {
                var type = req.params["providerId"] + "/" + req.params["resourceType"];
                var resourceId = req.params["resourceId"];
                var armManager = req.app.armManager;
                var resourceData = armManager.getResourceData(mockSubscriptionId, mockResourceGroupId, type, resourceId);
                if (resourceData) {
                    var resource = armManager.getResource(mockSubscriptionId, mockResourceGroupId, type, resourceId);
                    res.status(constants_1.Constants.HttpStatusCode.Ok).json(mapLocks(resource.id, resourceData.locks));
                }
                else {
                    res.sendStatus(constants_1.Constants.HttpStatusCode.NotFound);
                }
            }
            else {
                next();
            }
        });
        var resourceLockWithIdUri = resourceLocksUri + "/:id";
        app.put(resourceLockWithIdUri, function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            var mockResourceGroupId = req[urlParams_1.UrlParams.Params.resourceGroupId];
            if (mockSubscriptionId) {
                var id = req.params["id"].toLowerCase();
                var type = req.params["providerId"] + "/" + req.params["resourceType"];
                var resourceId = req.params["resourceId"];
                var armManager = req.app.armManager;
                var resourceData = armManager.getResourceData(mockSubscriptionId, mockResourceGroupId, type, resourceId);
                if (resourceData) {
                    var resource = armManager.getResource(mockSubscriptionId, mockResourceGroupId, type, resourceId);
                    var newLock = mapLock(resource.id, id, (req.body || {}).properties);
                    var locks = resourceData.locks || [];
                    resourceData.locks = updateLock(locks, newLock);
                    res.status(constants_1.Constants.HttpStatusCode.Ok).json(newLock);
                }
            }
            else {
                next();
            }
        });
    }
    Locks.init = init;
})(Locks = exports.Locks || (exports.Locks = {}));
//# sourceMappingURL=locks.js.map