"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Resources = void 0;
var armHelper_1 = require("../armHelper");
var constants_1 = require("../../Helpers/constants");
var odataHelper_1 = require("../../Helpers/odataHelper");
var urlParams_1 = require("./urlParams");
var Resources;
(function (Resources) {
    "use strict";
    /**
     * Initializes the routes.
     *
     * @param app The express app.
     */
    function init(app) {
        app.get("/resources", function (req, res, next) {
            var armManager = req.app.armManager;
            // TODO: cache this data once
            var allResources = [];
            // the list of actual resources returned by arm
            var actualResource = [];
            var processRequest = function () {
                armManager.getSubscriptions().forEach(function (sub) {
                    armManager.getResourceGroups(sub.subscriptionId).forEach(function (rg) {
                        var resources = armManager.getResources(sub.subscriptionId, rg.name).map(function (res) {
                            // expose properties that are using in OData filtering [name, resourcegroup, subscriptionid]
                            return {
                                name: res.name,
                                resourcegroup: rg.name,
                                subscriptionid: sub.subscriptionId,
                                mappedObj: {
                                    id: res.id,
                                    name: res.name,
                                    tags: res.tags,
                                    type: res.type,
                                    location: res.location,
                                    kind: res.kind
                                }
                            };
                        });
                        allResources = allResources.concat(resources);
                    });
                });
                var filter = req.query.$filter;
                var filteredResource = (filter ? odataHelper_1.ODataParser.evaluate("$filter=" + filter, allResources) : allResources).map(function (res) { return res.mappedObj; }) || [];
                // concat actual and mock resources and return data
                res.status(constants_1.Constants.HttpStatusCode.Ok).json({
                    value: filteredResource.concat(actualResource || [])
                });
            };
            armHelper_1.ArmHelper.handleRequest(req, res, processRequest, function (armManager) {
                armHelper_1.ArmHelper.proxyReq(req).then(function (resources) {
                    actualResource = resources;
                }).catch(function (err) {
                    // ignore errors
                }).finally(function () {
                    processRequest();
                });
            });
        });
        app.get("/subscriptions/:" + urlParams_1.UrlParams.Params.subscriptionId + "/resources", function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            if (mockSubscriptionId) {
                var armManager_1 = req.app.armManager;
                var resourceGroups = armManager_1.getResourceGroups(mockSubscriptionId);
                var resources = [].concat.apply([], resourceGroups.map(function (rg) {
                    return armManager_1.getResources(mockSubscriptionId, rg.name);
                }));
                res.status(constants_1.Constants.HttpStatusCode.Ok).json(resources);
            }
            else {
                next();
            }
        });
        app.get("/subscriptions/:" + urlParams_1.UrlParams.Params.subscriptionId + "/resourcegroups/:" + urlParams_1.UrlParams.Params.resourceGroupId + "/resources", function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            var mockResourceGroupId = req[urlParams_1.UrlParams.Params.resourceGroupId];
            if (mockSubscriptionId) {
                var armManager = req.app.armManager;
                var resources = armManager.getResources(mockSubscriptionId, mockResourceGroupId);
                res.status(constants_1.Constants.HttpStatusCode.Ok).json(resources);
            }
            else {
                next();
            }
        });
        app.route("/subscriptions/:" + urlParams_1.UrlParams.Params.subscriptionId + "/resourcegroups/:" + urlParams_1.UrlParams.Params.resourceGroupId + "/providers/:providerId/:resourceType/:resourceId")
            .get(function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            var mockResourceGroupId = req[urlParams_1.UrlParams.Params.resourceGroupId];
            if (mockSubscriptionId) {
                var armManager = req.app.armManager;
                var resource = armManager.getResource(mockSubscriptionId, mockResourceGroupId, req.params["providerId"] + "/" + req.params["resourceType"], req.params["resourceId"]);
                if (resource) {
                    res.status(constants_1.Constants.HttpStatusCode.Ok).json(resource);
                }
                else {
                    res.sendStatus(constants_1.Constants.HttpStatusCode.NotFound);
                }
            }
            else {
                next();
            }
        })
            .patch(function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            var mockResourceGroupId = req[urlParams_1.UrlParams.Params.resourceGroupId];
            if (mockSubscriptionId) {
                var armManager = req.app.armManager;
                var resource = armManager.getResourceData(mockSubscriptionId, mockResourceGroupId, req.params["providerId"] + "/" + req.params["resourceType"], req.params["resourceId"]);
                resource.tags = req.body;
                resource.mappedData = null;
                res.sendStatus(constants_1.Constants.HttpStatusCode.Ok);
            }
            else {
                next();
            }
        })
            .delete(function (req, res, next) {
            // TODO
            res.sendStatus(constants_1.Constants.HttpStatusCode.NotImplemented);
        });
        app.route("/subscriptions/:" + urlParams_1.UrlParams.Params.subscriptionId + "/resourcegroups/:" + urlParams_1.UrlParams.Params.resourceGroupId + "/providers/:providerId/:resourceType/:resourceId/:nestedResourceType/:nestedResourceId")
            .get(function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            var mockResourceGroupId = req[urlParams_1.UrlParams.Params.resourceGroupId];
            if (mockSubscriptionId) {
                var armManager = req.app.armManager;
                var resource = armManager.getResource(mockSubscriptionId, mockResourceGroupId, req.params["providerId"] + "/" + req.params["resourceType"] + "/" + req.params["nestedResourceType"], req.params["resourceId"] + "/" + req.params["nestedResourceId"]);
                if (resource) {
                    res.status(constants_1.Constants.HttpStatusCode.Ok).json(resource);
                }
                else {
                    res.sendStatus(constants_1.Constants.HttpStatusCode.NotFound);
                }
            }
            else {
                next();
            }
        })
            .patch(function (req, res, next) {
            var mockSubscriptionId = req[urlParams_1.UrlParams.Params.subscriptionId];
            var mockResourceGroupId = req[urlParams_1.UrlParams.Params.resourceGroupId];
            if (mockSubscriptionId) {
                var armManager = req.app.armManager;
                var resource = armManager.getNestedResourceData(mockSubscriptionId, mockResourceGroupId, req.params["providerId"] + "/" + req.params["resourceType"], req.params["resourceId"], req.params["nestedResourceType"], req.params["nestedResourceId"]);
                resource.tags = req.body;
                resource.mappedData = null;
                res.sendStatus(constants_1.Constants.HttpStatusCode.Ok);
            }
            else {
                next();
            }
        })
            .delete(function (req, res, next) {
            // TODO
            res.sendStatus(constants_1.Constants.HttpStatusCode.NotImplemented);
        });
    }
    Resources.init = init;
})(Resources = exports.Resources || (exports.Resources = {}));
//# sourceMappingURL=resources.js.map