import * as Q from "q";
import { ArmManager } from "./armManager";
export declare module ArmHelper {
    type HandleRequestCallback = (armManager: ArmManager.Manager) => void;
    /**
     * Determines if the controller should handle the incoming request if mock/proxy is setup.
     * Triggers passed in callback or executes the next middleware.
     *
     * @param req The incoming request.
     * @param res The outgoing response.
     * @param next The next handler in express.
     * @param callback The handler to be called if the request should be handled by mock/proxy.
     */
    function handleRequest(req: any, res: any, next: any, callback: HandleRequestCallback): void;
    /**
     * Proxies request to target, and pipes failures back immediatly.
     *
     * @param req The incoming request.
     * @param res The outgoing response.
     * @param defaultVal If proxy is not setup will return this.
     * @return A promise that resolve with the content if the proxied request succeeds, and reject if the request failed.
     */
    function proxyReq<T>(req: any, res?: any, defaultVal?: T): Q.Promise<T>;
}
