"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArmHelper = void 0;
var Q = require("q");
var ajaxHelper_1 = require("../Helpers/ajaxHelper");
var ArmHelper;
(function (ArmHelper) {
    "use strict";
    /**
     * Determines if the controller should handle the incoming request if mock/proxy is setup.
     * Triggers passed in callback or executes the next middleware.
     *
     * @param req The incoming request.
     * @param res The outgoing response.
     * @param next The next handler in express.
     * @param callback The handler to be called if the request should be handled by mock/proxy.
     */
    function handleRequest(req, res, next, callback) {
        var app = req.app;
        var armManager = app.armManager;
        var proxyEnabled = !!req.app.proxy;
        if (proxyEnabled || armManager) {
            callback(armManager);
        }
        else {
            next();
        }
    }
    ArmHelper.handleRequest = handleRequest;
    /**
     * Proxies request to target, and pipes failures back immediatly.
     *
     * @param req The incoming request.
     * @param res The outgoing response.
     * @param defaultVal If proxy is not setup will return this.
     * @return A promise that resolve with the content if the proxied request succeeds, and reject if the request failed.
     */
    function proxyReq(req, res, defaultVal) {
        var app = req.app;
        var proxy = app.proxy;
        if (!proxy) {
            return Q.resolve(defaultVal);
        }
        // get target from the proxy
        var host = proxy.options.target.host || proxy.options.target.replace("https://", "").replace("/", "");
        var defered = Q.defer();
        ajaxHelper_1.AjaxHelper.fwdAjax(req, host).then(function (response) {
            return ajaxHelper_1.AjaxHelper.getJsonData(response).then(function (body) {
                if (response.statusCode >= 400) {
                    if (res) {
                        // pipe failues back immediatly
                        res.status(response.statusCode).send(body);
                    }
                    defered.reject(body);
                }
                else {
                    defered.resolve((body && body.value) || body);
                }
            });
        });
        return defered.promise;
    }
    ArmHelper.proxyReq = proxyReq;
})(ArmHelper = exports.ArmHelper || (exports.ArmHelper = {}));
//# sourceMappingURL=armHelper.js.map