"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FirewallHelper = void 0;
var Q = require("q");
var FirewallHelper;
(function (FirewallHelper) {
    /**
     * Adds a firewall rule to open up a local port for node porxy server.
     * NOTE: Should be compiled with COMMENTS as this is required to execute PS scripts.
     *
     * @param name The name of the rule.
     * @param port The local port that is
     * @param overwrite Overwrite existing rule.
     * @return A promise that resolves when the rule is created.
     */
    function addFirewallRule(name, port, overwrite) {
        if (overwrite === void 0) { overwrite = true; }
        if (port > 65535) {
            return Q.reject("Port out of range.");
        }
        var defered = Q.defer();
        removeFirewallRule(name).catch(function () {
            // ignore errors
        }).then(function () {
            var exec = require('child_process').exec;
            var cmd = "netsh advfirewall firewall add rule name=\"" + name + "\" dir=in program=\"" + process.execPath + "\" protocol=tcp localport=" + port + " action=allow";
            exec(cmd, function (error, stdout, stderr) {
                if (error) {
                    defered.reject("Failed to overwrite existing firewall rule.\nError:" + error + ",\nSTDOUT:" + stdout + ",\nSTDERR:" + stderr);
                }
                else {
                    defered.resolve();
                }
            });
        });
        return defered.promise;
    }
    FirewallHelper.addFirewallRule = addFirewallRule;
    /**
     * Removes a firewall rule.
     * NOTE: Should be compiled with COMMENTS as this is required to eecute PS scripts.
     *
     * @param name The name of the rule.
     * @return A promise that resolves when the rule is removed.
     */
    function removeFirewallRule(name) {
        var defered = Q.defer();
        var exec = require('child_process').exec;
        var cmd = "netsh advfirewall firewall delete rule name=\"" + name + "\"";
        exec(cmd, function (error, stdout, stderr) {
            if (error) {
                defered.reject("Failed to remove existing firewall rule.\nError:" + error + ",\nSTDOUT:" + stdout + ",\nSTDERR:" + stderr);
            }
            else {
                defered.resolve();
            }
        });
        return defered.promise;
    }
    FirewallHelper.removeFirewallRule = removeFirewallRule;
})(FirewallHelper = exports.FirewallHelper || (exports.FirewallHelper = {}));
//# sourceMappingURL=firewallHelper.js.map