"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MockFx = void 0;
var bodyparser = require("body-parser");
var compression = require("compression");
var cors = require("cors");
var express = require("express");
var helmet = require("helmet");
var client_1 = require("./client");
var server_1 = require("./server");
var system_1 = require("./system");
var targets_1 = require("./targets");
var logger_1 = require("./logger");
var version = require("../../../package.json").version;
var MockFx = /** @class */ (function () {
    function MockFx(options, certificate) {
        var _this = this;
        this.proxies = {};
        this.tests = [];
        this.app = express();
        this.port = options.port;
        this.certificate = certificate;
        this.targetDefinitions = options.targets.map(function (target) {
            // Normalize strings
            target.name = target.name.toUpperCase();
            // Remove protocol and trailing slash if present
            target.hosts = target.hosts.map(_this.normalizeHost);
            return target;
        });
        this.registerBaseRoutes();
        this.registerPluginRoute();
        this.registerRegisterRoute();
        this.registerRegisterStageRoute();
        this.registerFinalizeRoute();
        this.registerFinalizeStageRoute();
        this.registerRequestHandler();
        this.registerErrorHandler();
        logger_1.logger.debug("Created MockFx instance for port " + this.port + " with targets " + JSON.stringify(this.targetDefinitions.map(function (target) { return target.name; })));
    }
    /**
     * Creates an instance of the framework
     *
     * @static
     * @param {MockFx.Configuration} config
     * @returns {Promise<MockFx>}
     * @memberof MockFx
     */
    MockFx.create = function (config) {
        return __awaiter(this, void 0, void 0, function () {
            var certificate;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!MockFx.loggingToConsole && config.logToConsole) {
                            MockFx.logToConsole();
                        }
                        logger_1.logger.debug("Creating new MockFx instance");
                        return [4 /*yield*/, system_1.getSSLCertificate()];
                    case 1:
                        certificate = _a.sent();
                        return [2 /*return*/, new MockFx({
                                logToConsole: config.logToConsole,
                                port: config.port,
                                targets: config.targets,
                            }, certificate)];
                }
            });
        });
    };
    /**
     * Returns an object with endpoints for the mock server for registering, finalizing, and obtaining the plug-in
     * code for a test.
     *
     * If using the plug-in endpoint with Portal's `patch` parameter, run the endpoint value through `encodeURI`
     *
     * @param {string} runId
     * @param {string} testId
     * @returns {MockFx.Endpoints}
     * @memberof MockFx
     */
    MockFx.getEndpoints = function (port, runId, testId) {
        var encodedRunId = encodeURIComponent(runId);
        var encodedTestId = encodeURIComponent(testId);
        return {
            finalizeStage: "https://localhost:" + port + "/mock/finalizeStage",
            finalizeTest: "https://localhost:" + port + "/mock/finalizeTest",
            plugin: "https://localhost:" + port + "/mock/plugin/runId/" + encodedRunId + "/testId/" + encodedTestId,
            registerStage: "https://localhost:" + port + "/mock/registerStage",
            registerTest: "https://localhost:" + port + "/mock/registerTest",
        };
    };
    MockFx.logToConsole = function () {
        MockFx.logger.on("log" /* Log */, function (level, message) { return console.log("MockFx", level, message); });
        MockFx.logger.on("error" /* Error */, function (error) { return console.error("MockFx", error); });
        MockFx.logger.on("telemetry" /* Telemetry */, function (telemetry) { return console.log("MockFx", "telemetry", telemetry); });
        MockFx.loggingToConsole = true;
    };
    /**
     * Starts the mock server
     *
     * @returns {Promise<void>}
     * @memberof MockFx
     */
    MockFx.prototype.start = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this;
                        return [4 /*yield*/, server_1.createHTTPSServer(this.app, server_1.getServerOptions(this.certificate), this.port)];
                    case 1:
                        _a.server = _b.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Stops the mock server
     *
     * @returns {Promise<void>}
     * @memberof MockFx
     */
    MockFx.prototype.stop = function () {
        return __awaiter(this, void 0, void 0, function () {
            var error_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.server) return [3 /*break*/, 5];
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, server_1.stopServer(this.server)];
                    case 2:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _a.sent();
                        logger_1.logger.error(error_1);
                        return [3 /*break*/, 4];
                    case 4:
                        this.server = null;
                        _a.label = 5;
                    case 5:
                        logger_1.logger.debug('Stopping proxy connections');
                        try {
                            Object.values(this.proxies).forEach(function (proxy) { return proxy.close(); });
                        }
                        catch (error) {
                            logger_1.logger.error(error);
                        }
                        this.proxies = {};
                        logger_1.logger.debug('Clearing tests and target definitions');
                        this.tests = [];
                        this.targetDefinitions = [];
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Finalizes a stage within a test. During replay, removes any subsequent requests that were tagged with this stage
     *
     * Also accessible over HTTPS POST at `MockFx.getEndpoints(mockFxPort, runId, testId).finalizeStage` end-point
     * by sending the stage context object as the body
     *
     * @param {MockFx.Stage} stageContext
     * @returns {Promise<void>}
     * @memberof MockFx
     */
    MockFx.prototype.finalizeStage = function (stageContext) {
        return __awaiter(this, void 0, void 0, function () {
            var testInstance, error;
            return __generator(this, function (_a) {
                testInstance = this.getTestInstance(stageContext.runId, stageContext.testId);
                if (!testInstance) {
                    error = new Error("Test has not been registered.");
                    logger_1.logger.error(error);
                    throw error;
                }
                testInstance.context.currentStage = "unset";
                if (testInstance.context.mode === "replay" /* Replay */) {
                    Object.values(testInstance.targets).map(function (target) {
                        target.removeStage(stageContext.stage);
                    });
                }
                logger_1.logger.finalizeStage(stageContext);
                return [2 /*return*/];
            });
        });
    };
    /**
     * Calls `storeRequests` on each target the test has registered, then removes the references to the instance
     * to free memory.
     *
     * Also accessible over HTTPS POST at `MockFx.getEndpoints(mockFxPort, runId, testId).finalizeTest` end-point
     * by sending the finalization object as the body
     *
     * @param {MockFx.Finalization} finalization
     * @returns {Promise<void>}
     * @memberof MockFx
     */
    MockFx.prototype.finalizeTest = function (finalization) {
        return __awaiter(this, void 0, void 0, function () {
            var testInstance, error, shouldStore, error_2;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        logger_1.logger.info("Finalizing test: " + finalization.runId + ";" + finalization.testId);
                        testInstance = this.getTestInstance(finalization.runId, finalization.testId);
                        if (!testInstance) {
                            error = new Error("Test has not been registered.");
                            logger_1.logger.error(error);
                            throw error;
                        }
                        shouldStore = typeof finalization.shouldStore === "boolean" ? finalization.shouldStore : true;
                        if (!(testInstance.context.mode === "record" /* Record */ && shouldStore)) return [3 /*break*/, 4];
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, Promise.all(Object.values(testInstance.targets).map(function (target) {
                                logger_1.logger.debug("Calling " + target.getDefinition().name + ".storeRequests");
                                return target.storeRequests();
                            }))];
                    case 2:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        error_2 = _a.sent();
                        logger_1.logger.error(error_2);
                        return [3 /*break*/, 4];
                    case 4:
                        // Remove test reference
                        this.tests = this.tests.filter(function (instance) { return !(instance.context.runId === finalization.runId && instance.context.testId === finalization.testId); });
                        logger_1.logger.debug("Finalized test: " + finalization.runId + ";" + finalization.testId);
                        logger_1.logger.finalizeTest(finalization);
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Returns the mock plug-in client code to run in the browser
     *
     * Also accessible over HTTPS GET at `MockFx.getEndpoints(mockFxPort, runId, testId).plugin` end-point
     *
     * @param {string} runId
     * @param {string} testId
     * @returns {string}
     * @memberof MockFx
     */
    MockFx.prototype.getPluginCode = function (runId, testId) {
        var testInstance = this.getTestInstance(runId, testId);
        if (!testInstance) {
            var error = new Error("Test has not been registered.");
            logger_1.logger.error(error);
            throw error;
        }
        var targetDefinitions = Object.values(testInstance.targets).map(function (target) { return target.getDefinition(); });
        return client_1.getPlugin(targetDefinitions, runId, testId, this.port);
    };
    /**
     * Registers a stage within a test. All requests occurring for a test will be tagged with the current stage.
     * Used if parts of a test (a stage) are conditional or can be skipped.
     *
     * Also accessible over HTTPS POST at `MockFx.getEndpoints(mockFxPort, runId, testId).registerStage` end-point
     * by sending the stage context object as the body
     *
     * @param {MockFx.Stage} stageContext
     * @returns {Promise<void>}
     * @memberof MockFx
     */
    MockFx.prototype.registerStage = function (stageContext) {
        return __awaiter(this, void 0, void 0, function () {
            var testInstance, error;
            return __generator(this, function (_a) {
                testInstance = this.getTestInstance(stageContext.runId, stageContext.testId);
                if (!testInstance) {
                    error = new Error("Test has not been registered.");
                    logger_1.logger.error(error);
                    throw error;
                }
                testInstance.context.currentStage = stageContext.stage;
                if (testInstance.context.mode === "replay" /* Replay */) {
                    Object.values(testInstance.targets).map(function (target) {
                        target.snapToStage(stageContext.stage);
                    });
                }
                logger_1.logger.registerStage(stageContext);
                return [2 /*return*/];
            });
        });
    };
    /**
     * Registers a new test, creates instances for each target referenced, and loads their test data if in
     * replay mode.
     *
     * Also accessible over HTTPS POST at `MockFx.getEndpoints(mockFxPort, runId, testId).registerTest` end-point
     * by sending the context object as the body
     *
     * @param {MockFx.Context} context
     * @returns {Promise<void>}
     * @memberof MockFx
     */
    MockFx.prototype.registerTest = function (context) {
        return __awaiter(this, void 0, void 0, function () {
            var fullContext, testInstance, targets, _loop_1, this_1, _i, _a, target;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        fullContext = __assign(__assign({}, context), { proxyUnmatched: context.proxyUnmatched ? context.proxyUnmatched : false, metadata: context.metadata !== undefined ? context.metadata : {}, currentStage: "initial" });
                        logger_1.logger.info("Registering test: " + fullContext.runId + ";" + fullContext.testId);
                        testInstance = this.getTestInstance(fullContext.runId, fullContext.testId);
                        if (testInstance) {
                            logger_1.logger.warn('Test was already registered.');
                            return [2 /*return*/];
                        }
                        targets = {};
                        _loop_1 = function (target) {
                            var definition = this_1.targetDefinitions.find(function (definition) { return definition.name === target.toUpperCase(); });
                            if (definition) {
                                targets[definition.name] = new targets_1.Target(definition, fullContext);
                            }
                            else {
                                var error = new Error("Target \"" + target + "\" has not been registered.");
                                logger_1.logger.error(error);
                                throw error;
                            }
                        };
                        this_1 = this;
                        for (_i = 0, _a = fullContext.targets; _i < _a.length; _i++) {
                            target = _a[_i];
                            _loop_1(target);
                        }
                        // Notify each target registered
                        return [4 /*yield*/, Promise.all(Object.values(targets).map(function (target) {
                                logger_1.logger.debug("Calling " + target.getDefinition().name + ".onRegister");
                                return target.onRegister();
                            }))];
                    case 1:
                        // Notify each target registered
                        _b.sent();
                        if (!(fullContext.mode === "replay" /* Replay */)) return [3 /*break*/, 3];
                        // Load requests for each target
                        return [4 /*yield*/, Promise.all(Object.values(targets).map(function (target) {
                                logger_1.logger.debug("Calling " + target.getDefinition().name + ".loadRequests");
                                return target.loadRequests();
                            }))];
                    case 2:
                        // Load requests for each target
                        _b.sent();
                        _b.label = 3;
                    case 3:
                        this.tests.push({
                            context: fullContext,
                            targets: targets,
                        });
                        logger_1.logger.debug("Registered test: " + fullContext.runId + ";" + fullContext.testId);
                        logger_1.logger.registerTest(fullContext);
                        return [2 /*return*/];
                }
            });
        });
    };
    MockFx.prototype.getTestInstance = function (runId, testId) {
        return this.tests.find(function (instance) { return instance.context.runId === runId && instance.context.testId === testId; });
    };
    MockFx.prototype.registerBaseRoutes = function () {
        this.app.use(helmet());
        this.app.use(cors());
        this.app.use(compression());
        this.app.use(bodyparser.json());
    };
    MockFx.prototype.registerPluginRoute = function () {
        var _this = this;
        /**
         * @swagger
         * /mock/plugin/runId/{runId}/testId/{testId}:
         *   get:
         *     summary: "Gets the mock plug-in client code for a test."
         *     produces:
         *     - "test/javascript"
         *     parameters:
         *     - name: "runId"
         *       in: "path"
         *       description: "The run ID of the test"
         *       required: true
         *       type: "string"
         *     - name: "testId"
         *       in: "path"
         *       description: "The test ID of the test"
         *       required: true
         *       type: "string"
         *     responses:
         *       "200":
         *         description: "Mock plug-in client code"
         */
        this.app.get("/mock/plugin/runId/:runId/testId/:testId", function (req, res, next) { return __awaiter(_this, void 0, void 0, function () {
            var plugin, err_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        logger_1.logger.debug("Serving plug-in code for " + req.params.runId + ";" + req.params.testId);
                        if (typeof req.params.runId !== "string" || req.params.runId.length === 0) {
                            throw new Error("\"runId\" parameter was not provided.");
                        }
                        if (typeof req.params.testId !== "string" || req.params.testId.length === 0) {
                            throw new Error("\"testId\" parameter was not provided.");
                        }
                        return [4 /*yield*/, this.getPluginCode(req.params.runId, req.params.testId)];
                    case 1:
                        plugin = _a.sent();
                        return [2 /*return*/, res.contentType("text/javascript").send(plugin)];
                    case 2:
                        err_1 = _a.sent();
                        next(err_1);
                        return [3 /*break*/, 3];
                    case 3: return [2 /*return*/];
                }
            });
        }); });
    };
    MockFx.prototype.registerErrorHandler = function () {
        this.app.use(function (err, req, res, next) {
            logger_1.logger.error(err);
            res.status(500).send(err.message);
        });
    };
    MockFx.prototype.registerFinalizeRoute = function () {
        var _this = this;
        /**
         * @swagger
         * /mock/finalizeTest:
         *   post:
         *     summary: "Finalizes the test run."
         *     consumes:
         *     - "application/json"
         *     parameters:
         *     - name: "runId"
         *       in: "body"
         *       description: "The finalization parameters"
         *       required: true
         *       schema:
         *         $ref: "#/definitions/FinalizeTest"
         *     responses:
         *       "200":
         *         description: "successful operation"
         */
        this.app.post("/mock/finalizeTest", function (req, res, next) { return __awaiter(_this, void 0, void 0, function () {
            var body, error_3;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        logger_1.logger.debug("Processing finalization request for " + req.body.runId + ";" + req.body.testId);
                        body = req.body;
                        if (!body) {
                            throw new Error("Finalize object was not given.");
                        }
                        if (typeof body.runId !== "string" || body.runId.length === 0) {
                            throw new Error('Valid "runId" property was not provided.');
                        }
                        if (typeof body.testId !== "string" || body.testId.length === 0) {
                            throw new Error('Valid "testId" property was not provided.');
                        }
                        return [4 /*yield*/, this.finalizeTest({
                                runId: body.runId,
                                testId: body.testId,
                                shouldStore: body.shouldStore
                            })];
                    case 1:
                        _a.sent();
                        res.sendStatus(200);
                        return [3 /*break*/, 3];
                    case 2:
                        error_3 = _a.sent();
                        next(error_3);
                        return [3 /*break*/, 3];
                    case 3: return [2 /*return*/];
                }
            });
        }); });
    };
    MockFx.prototype.registerFinalizeStageRoute = function () {
        var _this = this;
        /**
         * @swagger
         * /mock/finalizeStage:
         *   post:
         *     summary: "Finalizes a test's stage."
         *     consumes:
         *     - "application/json"
         *     parameters:
         *     - in: "body"
         *       description: "The finalization parameters"
         *       required: true
         *       schema:
         *         $ref: "#/definitions/Stage"
         *     responses:
         *       "200":
         *         description: "successful operation"
         */
        this.app.post("/mock/finalizeStage", function (req, res, next) { return __awaiter(_this, void 0, void 0, function () {
            var body, error_4;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        logger_1.logger.debug("Processing stage finalization request for " + req.body.runId + ";" + req.body.testId + ";" + req.body.stage);
                        body = req.body;
                        if (!body) {
                            throw new Error("Finalize stage object was not given.");
                        }
                        if (typeof body.runId !== "string" || body.runId.length === 0) {
                            throw new Error('Valid "runId" property was not provided.');
                        }
                        if (typeof body.testId !== "string" || body.testId.length === 0) {
                            throw new Error('Valid "testId" property was not provided.');
                        }
                        if (typeof body.stage !== "string" || body.stage.length === 0) {
                            throw new Error('Valid "stage" property was not provided.');
                        }
                        return [4 /*yield*/, this.finalizeStage({
                                runId: body.runId,
                                testId: body.testId,
                                stage: body.stage
                            })];
                    case 1:
                        _a.sent();
                        res.sendStatus(200);
                        return [3 /*break*/, 3];
                    case 2:
                        error_4 = _a.sent();
                        next(error_4);
                        return [3 /*break*/, 3];
                    case 3: return [2 /*return*/];
                }
            });
        }); });
    };
    MockFx.prototype.registerRegisterRoute = function () {
        var _this = this;
        var targetNames = this.targetDefinitions.map(function (target) { return target.name; });
        /**
         * @swagger
         * /mock/registerTest:
         *   post:
         *     summary: "Registers a test run."
         *     consumes:
         *     - "application/json"
         *     parameters:
         *     - name: "body"
         *       in: "body"
         *       description: "The MockFx.Context for the test run"
         *       required: true
         *       schema:
         *         $ref: "#/definitions/Context"
         *     responses:
         *       "200":
         *         description: "successful operation"
         */
        this.app.post("/mock/registerTest", function (req, res, next) { return __awaiter(_this, void 0, void 0, function () {
            var context, error_5;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        logger_1.logger.debug("Processing registration request for " + req.body.runId + ";" + req.body.testId);
                        context = req.body;
                        if (!context) {
                            throw new Error("Context object was not given.");
                        }
                        if (typeof context.runId !== "string" || context.runId.length === 0) {
                            throw new Error('Valid "runId" property was not provided.');
                        }
                        if (typeof context.testId !== "string" || context.testId.length === 0) {
                            throw new Error('Valid "testId" property was not provided.');
                        }
                        if (typeof context.mode !== "string" ||
                            (context.mode !== "record" /* Record */ && context.mode !== "replay" /* Replay */)) {
                            throw new Error('Valid "mode" property was not provided.');
                        }
                        if (!Array.isArray(context.targets) ||
                            context.targets.length === 0 ||
                            !context.targets.every(function (target) { return targetNames.includes(target.toUpperCase()); })) {
                            throw new Error('Invalid "targets" property was provided.');
                        }
                        return [4 /*yield*/, this.registerTest(context)];
                    case 1:
                        _a.sent();
                        res.sendStatus(200);
                        return [3 /*break*/, 3];
                    case 2:
                        error_5 = _a.sent();
                        next(error_5);
                        return [3 /*break*/, 3];
                    case 3: return [2 /*return*/];
                }
            });
        }); });
    };
    MockFx.prototype.registerRegisterStageRoute = function () {
        var _this = this;
        /**
         * @swagger
         * /mock/registerStage:
         *   post:
         *     summary: "Registers a test's stage."
         *     consumes:
         *     - "application/json"
         *     parameters:
         *     - in: "body"
         *       description: "The registration parameters"
         *       required: true
         *       schema:
         *         $ref: "#/definitions/Stage"
         *     responses:
         *       "200":
         *         description: "successful operation"
         */
        this.app.post("/mock/registerStage", function (req, res, next) { return __awaiter(_this, void 0, void 0, function () {
            var body, error_6;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        logger_1.logger.debug("Processing stage registration request for " + req.body.runId + ";" + req.body.testId + ";" + req.body.stage);
                        body = req.body;
                        if (!body) {
                            throw new Error("Stage object was not given.");
                        }
                        if (typeof body.runId !== "string" || body.runId.length === 0) {
                            throw new Error('Valid "runId" property was not provided.');
                        }
                        if (typeof body.testId !== "string" || body.testId.length === 0) {
                            throw new Error('Valid "testId" property was not provided.');
                        }
                        if (typeof body.stage !== "string" || body.stage.length === 0) {
                            throw new Error('Valid "stage" property was not provided.');
                        }
                        return [4 /*yield*/, this.registerStage({
                                runId: body.runId,
                                testId: body.testId,
                                stage: body.stage
                            })];
                    case 1:
                        _a.sent();
                        res.sendStatus(200);
                        return [3 /*break*/, 3];
                    case 2:
                        error_6 = _a.sent();
                        next(error_6);
                        return [3 /*break*/, 3];
                    case 3: return [2 /*return*/];
                }
            });
        }); });
    };
    MockFx.prototype.registerRequestHandler = function () {
        var _this = this;
        // Prefixes incoming requests with their matching target name to route to the right handler with Express
        this.app.use(function (req, res, next) {
            try {
                var originalTarget_1 = req.headers["x-mockfx-original-host" /* ORIGINAL_HOST */];
                var runId = req.headers["x-mockfx-run-id" /* RUN_ID */];
                var testId = req.headers["x-mockfx-test-id" /* TEST_ID */];
                var testInstance = _this.getTestInstance(runId, testId);
                if (!testInstance) {
                    throw new Error("Test has not been registered.");
                }
                if (originalTarget_1) {
                    /**
                     * For each target registered in the test, in the order it was registered, find the first
                     * target where its `hosts` array contains a host which is a substring of the request's original
                     * target host.
                     *
                     * For an original target of https://management.azure.com/, a target with a host of
                     * "management.azure.com" would match
                     */
                    var matchedTarget = Object.values(testInstance.targets).find(function (target) {
                        return target.getDefinition().hosts.some(function (host) { return _this.normalizeHost(originalTarget_1).includes(host); });
                    });
                    if (matchedTarget) {
                        req.headers["x-mockfx-target" /* TARGET */] = matchedTarget.getDefinition().name;
                        req.url = "/" + matchedTarget.getDefinition().name + req.url;
                        return next();
                    }
                }
                next(new Error("Target not found."));
            }
            catch (error) {
                next(error);
            }
        });
        // Register target route handlers
        for (var _i = 0, _a = this.targetDefinitions; _i < _a.length; _i++) {
            var target = _a[_i];
            if (target.getRouter) {
                var routes = target.getRouter();
                if (routes) {
                    this.app.use("/" + target.name, routes);
                }
            }
        }
        this.app.use(function (req, res, next) { return __awaiter(_this, void 0, void 0, function () {
            var runId, testId, originalHost, targetName, normalizedHost, testInstance, target, proxy, metadataCb, parseResponseCb, error_7;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        runId = req.headers["x-mockfx-run-id" /* RUN_ID */];
                        testId = req.headers["x-mockfx-test-id" /* TEST_ID */];
                        originalHost = req.headers["x-mockfx-original-host" /* ORIGINAL_HOST */];
                        targetName = req.headers["x-mockfx-target" /* TARGET */];
                        if (!runId || !testId || !originalHost || !targetName) {
                            throw new Error("Invalid headers sent.");
                        }
                        normalizedHost = this.normalizeHost(originalHost);
                        testInstance = this.getTestInstance(runId, testId);
                        if (!testInstance) {
                            throw new Error("Test has not been registered.");
                        }
                        target = testInstance.targets[targetName];
                        if (!target) {
                            throw new Error("Test was not registered for \"" + target.getDefinition().name + "\" target.");
                        }
                        proxy = this.proxies[normalizedHost];
                        if (!proxy) {
                            logger_1.logger.debug("Creating new proxy for " + target.getDefinition().name + ";" + normalizedHost);
                            metadataCb = target.getDefinition().getRequestMetadata
                                ? target.getDefinition().getRequestMetadata
                                : function () { return Promise.resolve({}); };
                            parseResponseCb = target.getDefinition().parseResponse
                                ? target.getDefinition().parseResponse
                                : function (res, body) { return Promise.resolve(body.toString("utf8")); };
                            this.proxies[normalizedHost] = proxy = server_1.createProxy(normalizedHost, server_1.getServerOptions(this.certificate), metadataCb, parseResponseCb, function (request, mockRequest) { return _this.handleProxyResponse(request, mockRequest); });
                        }
                        return [4 /*yield*/, target.handleRequest(proxy.send, req, res, next)];
                    case 1:
                        _a.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        error_7 = _a.sent();
                        next(error_7);
                        return [3 /*break*/, 3];
                    case 3: return [2 /*return*/];
                }
            });
        }); });
    };
    MockFx.prototype.handleProxyResponse = function (request, mockRequest) {
        return __awaiter(this, void 0, void 0, function () {
            var runId, testId, targetName, testInstance, target, error_8;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        runId = request.headers["x-mockfx-run-id" /* RUN_ID */];
                        testId = request.headers["x-mockfx-test-id" /* TEST_ID */];
                        targetName = request.headers["x-mockfx-target" /* TARGET */];
                        if (!runId || !testId || !targetName) {
                            throw new Error("Invalid headers sent.");
                        }
                        testInstance = this.getTestInstance(runId, testId);
                        if (!testInstance) {
                            throw new Error("Test has not been registered.");
                        }
                        target = testInstance.targets[targetName];
                        if (!target) {
                            throw new Error("Test was not registered for \"" + target.getDefinition().name + "\" target.");
                        }
                        return [4 /*yield*/, target.handleProxyResponse(request, mockRequest)];
                    case 1:
                        _a.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        error_8 = _a.sent();
                        logger_1.logger.error(error_8);
                        throw error_8;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    MockFx.prototype.normalizeHost = function (host) {
        return host.replace(/^(https?:\/\/)|\/$/gi, "").toUpperCase();
    };
    /**
     * The current version of the framework. It's a good idea to use this to bin your stored mock data
     * to allow for upgrading the framework when breaking changes are introduced in order to not break
     * test runs using previous version.
     *
     * @static
     * @memberof MockFx
     */
    MockFx.version = version;
    /**
     * Log event emitter. May subscribe to "log", "telemetry" and "error" events.
     *
     * @static
     * @type {LoggerType}
     * @memberof MockFx
     */
    MockFx.logger = logger_1.logger;
    MockFx.loggingToConsole = false;
    return MockFx;
}());
exports.MockFx = MockFx;
(function (MockFx) {
})(MockFx = exports.MockFx || (exports.MockFx = {}));
exports.MockFx = MockFx;
//# sourceMappingURL=index.js.map