Param(
    [string]$FriendlyName
)

$ErrorActionPreference = "Stop"

$RootName = "$($FriendlyName) Root"

Function New-Certificate {
    $begin = Get-Date
    $end = $begin.AddDays(90)

    # Create root certificate
    $rootCA = New-SelfSignedCertificate `
        -DnsName $RootName `
        -KeyLength 2048 `
        -KeyAlgorithm 'RSA' `
        -HashAlgorithm 'SHA256' `
        -KeyExportPolicy 'Exportable' `
        -NotBefore $begin `
        -NotAfter $end `
        -CertStoreLocation 'Cert:\LocalMachine\My' `
        -KeyUsage 'CertSign','CRLSign'

    # Add certificate to root store
    $certStore = New-Object System.Security.Cryptography.X509Certificates.X509Store 'Root', 'LocalMachine'
    $certStore.Open('ReadWrite')
    try {
        $certStore.Add($rootCA)
    }
    finally {
        $certStore.Close();
    }

    # Create certificate
    $cert = New-SelfSignedCertificate `
        -DnsName "localhost" `
        -Signer $rootCA `
        -KeyLength 2048 `
        -KeyAlgorithm 'RSA' `
        -HashAlgorithm 'SHA256' `
        -KeyExportPolicy 'Exportable' `
        -NotBefore $begin `
        -NotAfter $end `
        -CertStoreLocation 'Cert:\LocalMachine\My' `
        -FriendlyName $FriendlyName

    return $cert
}

Function Get-RootCertificates {
    return Get-ChildItem 'Cert:\LocalMachine' -Recurse | Where-Object { $_.Subject -eq "CN=$($RootName)" }
}

Function Get-Certificates {
    return Get-ChildItem 'Cert:\LocalMachine' -Recurse | Where-Object { $_.FriendlyName -eq $FriendlyName }
}

# Find certificates that are not about to expire
$certificates = Get-Certificates | Where-Object { $_.NotAfter -gt (Get-Date).AddDays((1)) }
$rootCertificates = Get-RootCertificates | Where-Object { $_.NotAfter -gt (Get-Date).AddDays((1)) }

# If we don't have the correct number of certificates, refresh everything
if ($certificates.length -ne 1 -or $rootCertificates.length -ne 3) {
    Get-Certificates | Remove-Item
    Get-RootCertificates | Remove-Item

    $certificate = New-Certificate
} else {
    $certificate = $certificates[0]
}

# Generate base64-encoded PFX
$pfxPassword = [System.Guid]::NewGuid().ToString()
$securePfxPassword = ConvertTo-SecureString -String $pfxPAssword -Force -AsPlainText
$pfx = $certificate.Export([System.Security.Cryptography.X509Certificates.X509ContentType]::Pkcs12, $securePfxPassword)

return "$([System.Convert]::ToBase64String($pfx))^$($pfxPassword)"