"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateDeploymentNames = exports.isDeploymentRequest = exports.deploymentRegex = void 0;
exports.deploymentRegex = /\/subscriptions\/.*?\/resourceGroups\/.*?\/providers\/Microsoft\.Resources\/deployments\/(.+?)(?:\/validate)?\?api-version=(\d\d\d\d-\d\d-\d\d)/i;
function isDeploymentRequest(req) {
    // Identify the portal specific ARM deployment calls
    if (exports.deploymentRegex.test(req.originalUrl) && req.method === 'GET' && req.headers.commandname && req.headers.commandname.toString().endsWith('.Deploy.getDepName')) {
        return true;
    }
    if (exports.deploymentRegex.test(req.originalUrl) && req.method === 'PUT' && req.headers.commandname && req.headers.commandname.toString().endsWith('.Deploy.submitDeployment')) {
        return true;
    }
    return false;
}
exports.isDeploymentRequest = isDeploymentRequest;
function updateDeploymentNames(requests, req, seenDeployments) {
    var deployment = requests.find(function (mockRequest) {
        if (mockRequest.metadata.command && mockRequest.metadata.command.endsWith(".Deploy.submitDeployment")) {
            var matches = mockRequest.url.match(exports.deploymentRegex);
            // We haven't seen this deployment request before
            return matches !== null && seenDeployments.indexOf(matches[1]) === -1;
        }
        return false;
    });
    var newDeploymentName = req.originalUrl.match(exports.deploymentRegex)[1];
    // We've already updated the requests for this deployment
    if (!deployment || seenDeployments.indexOf(newDeploymentName) !== -1) {
        return [requests, seenDeployments];
    }
    var oldDeploymentName = deployment.url.match(exports.deploymentRegex)[1];
    var json = JSON.stringify(requests);
    var newRequests = JSON.parse(replaceAll(json, oldDeploymentName, newDeploymentName));
    seenDeployments.push(oldDeploymentName);
    seenDeployments.push(newDeploymentName);
    return [newRequests, seenDeployments];
}
exports.updateDeploymentNames = updateDeploymentNames;
function replaceAll(input, searchValue, replaceValue) {
    return input.replace(new RegExp(regexEscape(searchValue), "g"), replaceValue);
}
function regexEscape(str) {
    return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
}
//# sourceMappingURL=helpers.js.map