"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.invokeRouter = void 0;
var axios_1 = require("axios");
var express_1 = require("express");
var url_1 = require("url");
var server_1 = require("../../server");
var xmsPathQuery = "x-ms-path-query";
exports.invokeRouter = express_1.Router();
exports.invokeRouter.all("/", function (req, res, next) {
    try {
        var host = req.get("host");
        var path = req.headers[xmsPathQuery];
        if (path.includes("://")) {
            var parsedUrl = url_1.parse(path);
            path = parsedUrl.href ? parsedUrl.href.replace(parsedUrl.protocol + "//" + parsedUrl.host, "") : "";
        }
        path = encodeRequestURL(path);
        var url = req.protocol + "://" + host + path;
        var method = req.method;
        var hasBody = req.headers['content-length'] !== undefined;
        var data = hasBody ? req.body : undefined;
        var headers = stripHeaders(!hasBody, __assign({}, server_1.parseRawHeaders(req.rawHeaders)));
        delete headers[xmsPathQuery];
        return axios_1.default({ url: url, method: method, headers: headers, data: data })
            .then(function (response) {
            res.status(response.status).send(response.data);
        })
            .catch(function (error) {
            next(error);
        });
    }
    catch (error) {
        next(error);
    }
});
function encodeRequestURL(url) {
    // Alphanumerics, -._~!$'()*+,;:@/? symbols, and percent-encoded characters (e.g. %25) do not need to be encoded in query parameters
    // If the parameter values only include these characters, no need to encode. Prevents encoding already-encoded URLs
    // https://stackoverflow.com/a/49400367
    var validQueryParameterValue = /^([a-zA-Z0-9\-._~!$'()*+,;:@/?]|%[a-fA-F0-9]{2})*$/;
    var path = url.indexOf("?") !== -1 ? url.substr(0, url.indexOf("?")) : url;
    var query = url.indexOf("?") !== -1 ? url.substr(url.indexOf("?") + 1) : "";
    if (query) {
        var params = query
            .split("&")
            .map(function (param) {
            var name = param.substr(0, param.indexOf("="));
            var value = param.substr(param.indexOf("=") + 1);
            return name + "=" + (validQueryParameterValue.test(value) ? value : encodeURIComponent(value));
        })
            .join("&");
        url = path + "?" + params;
    }
    return url;
}
function stripHeaders(stripType, headers) {
    var headerPrefixesToRemove = [
    // "Access-Control-", "Origin", "Sec-", "pragma", "Cache-control", "Connection"
    ];
    if (stripType) {
        headerPrefixesToRemove.push("Content-Type");
        headerPrefixesToRemove.push("Content-Length");
    }
    var validHeaders = Object.keys(headers).filter(function (header) {
        return !headerPrefixesToRemove.some(function (cors) { return header.toUpperCase().startsWith(cors.toUpperCase()); });
    });
    return validHeaders.reduce(function (obj, key) {
        obj[key] = headers[key];
        return obj;
    }, {});
}
//# sourceMappingURL=invoke.js.map