import * as Q from "q";
import * as RdfeModel from "./Models/rdfeModel";
import { StringMap } from "../Helpers/html5";
export declare module RdfeManager {
    /**
     * Defines structore of mock data.
     */
    interface MockData {
        /**
         * The list of subscriptions which will be available for Management Certificate operations.
         */
        subscriptions?: RdfeModel.Subscription[];
        /**
         * A map of SubscriptionId keys to SubscriptionCertificate lists which will be use to initialize the RdfeManager.
         */
        certificates?: StringMap<RdfeModel.SubscriptionCertificate[]>;
    }
    /**
     * The mock rdfe manager
     */
    class Manager {
        private data;
        /**
         * Initializes the mock data. Call this with empty data to reset state.
         *
         * @param data The mock data.
         * @return A promise that is resolved when the data is initialized.
         */
        initializeMockData(data: MockData): Q.Promise<any>;
        /**
         * Gets the list of mock subscriptions.
         */
        getSubscriptions(): RdfeModel.Subscription[];
        /**
         * Gets a subscription based on id from the mock data.
         *
         * @param subscriptionId The subscription GUID/ID.
         */
        getSubscription(subscriptionId: string): RdfeModel.Subscription;
        /**
         * The Add Management Certificate operation adds a management certificate to the specified subscription.
         *
         * @param subscriptionId The subscription GUID/ID.
         * @param certificate The SubscriptionCertificate to add.
         * @returns false if the subscription doesn't exist,
         *          false if the certificate already exists for the given subscription id,
         *          true if the certificate was successfully added.
         */
        addManagementCertificate(subscriptionId: string, subscriptionCertificate: RdfeModel.SubscriptionCertificate): boolean;
        /**
         * The Delete Management Certificate operation deletes a certificate from the specified subscription.
         *
         * @param subscriptionId The subscription GUID/ID.
         * @param thumbprint The thumbprint of the SubscriptionCertificate to delete.
         * @returns false if the subscription doesn't exist,
         *          false if the certificate doiesn't exist for the given subscription id,
         *          true if the certificate was successfully deleted.
         */
        deleteManagementCertificate(subscriptionId: string, thumbprint: string): boolean;
        /**
         * The Get Management Certificate operation retrieves information about the management certificate with the specified thumbprint.
         *
         * @param subscriptionId The subscription GUID/ID.
         * @param thumbprint The thumbprint of the SubscriptionCertificate to retrieve.
         */
        getManagementCertificate(subscriptionId: string, thumbprint: string): RdfeModel.SubscriptionCertificate;
        /**
         * The List Management Certificates operation returns basic information about all of the management certificates that are associated with the specified subscription.
         *
         * @param subscriptionId The subscription GUID/ID.
         */
        listManagementCertificates(subscriptionId: string): RdfeModel.SubscriptionCertificate[];
    }
}
