"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RdfeProxy = void 0;
var rdfeManager_1 = require("./rdfeManager");
var certHelper_1 = require("../Helpers/certHelper");
var firewallHelper_1 = require("../Helpers/firewallHelper");
var httpsProxy_1 = require("../HttpsProxy/httpsProxy");
var Certificates_1 = require("./Controllers/Certificates");
var urlParams_1 = require("./Controllers/urlParams");
var RdfeProxy;
(function (RdfeProxy) {
    "use strict";
    var firewallRuleName = "Azure Portal Rdfe Test Proxy";
    /**
     * Creates an rdfe proxy instance and setups firewal and certificates.
     *
     * @param rdfeEndpoint The rdfe endpoint to proxy.
     * @param serverPort The local server port.
     * @param rdfeManager The rdfe mock manager.
     * @param enablePassthrough If to enable passthrough to actual rdfe.
     * @return A promise that completes when local proxy is setup.
     */
    function create(rdfeEndpoint, serverPort, rdfeManager, customRoutes, enablePassthrough) {
        if (enablePassthrough === void 0) { enablePassthrough = true; }
        process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0";
        rdfeEndpoint = rdfeEndpoint.replace("https://", "").replace("/", "");
        return firewallHelper_1.FirewallHelper.addFirewallRule(firewallRuleName, serverPort, true).then(function () {
            return certHelper_1.CertHelper.createLocalHostSSLOptions().then(function (sslOptions) {
                var serverOptions = {
                    serverPort: serverPort
                };
                var serverPromise;
                if (enablePassthrough) {
                    serverPromise = httpsProxy_1.HttpsProxy.createServerWithProxy(sslOptions, rdfeEndpoint, serverOptions);
                }
                else {
                    serverPromise = httpsProxy_1.HttpsProxy.createServer(sslOptions, rdfeEndpoint, serverOptions);
                }
                return serverPromise.then(function (server) {
                    server.app.rdfeManager = rdfeManager || new rdfeManager_1.RdfeManager.Manager();
                    // register user routes first
                    if (customRoutes)
                        customRoutes(server.app);
                    initializeRoutes(server.app);
                    // set the default routes, SHOULD be registered after all other routes as express matches routes
                    // from first registered to last.
                    server.initializeDefaultRoutes();
                    return server;
                });
            });
        });
    }
    RdfeProxy.create = create;
    /**
     * Dispose local proxy
     *
     * @param rdfeEndpoint The rdfe endpoint to proxy.
     * @param serverPort The local server port.
     * @param enablePassthrough If to enable passthrough to actual rdfe.
     * @return A promise that completes when local proxy is setup.
     */
    function dispose(proxy) {
        return firewallHelper_1.FirewallHelper.removeFirewallRule(firewallRuleName).then(function () {
            proxy.server.close();
            if (proxy.proxy)
                proxy.proxy.close();
        });
    }
    RdfeProxy.dispose = dispose;
    function initializeRoutes(app) {
        urlParams_1.UrlParams.init(app);
        Certificates_1.Certificates.init(app);
    }
})(RdfeProxy = exports.RdfeProxy || (exports.RdfeProxy = {}));
//# sourceMappingURL=rdfeProxy.js.map